/*
    isprelink Python Extension Module

    Portions of the code in this file come from rpm-4.3.3

    Copyright (C) 2007  Mike Brady <mike.brady@devnull.net.nz>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Python.h>
#include <gelf.h>

#if !defined(DT_GNU_PRELINKED)
#define	DT_GNU_PRELINKED	0x6ffffdf5
#endif
#if !defined(DT_GNU_LIBLIST)
#define	DT_GNU_LIBLIST		0x6ffffef9
#endif


#if PY_VERSION_HEX < 0x02050000
typedef int Py_ssize_t;
#endif

static PyObject*
isprelink(PyObject* self, PyObject* args)
/*
 * Code in this function has been extracted from the RPM source code and comes
 * from the function open_dso() in the file rpmdb/legacy.c.
 */
{
    Elf *elf = NULL;
    Elf_Scn *scn = NULL;
    Elf_Data *data = NULL;
    GElf_Ehdr ehdr;
    GElf_Shdr shdr;
    GElf_Dyn dyn;
    int bingo = 0;
    int fd;

    (void) elf_version(EV_CURRENT);
    
    if (!PyArg_ParseTuple(args, "i", &fd)) return NULL;

    if ((elf = elf_begin (fd, ELF_C_READ, NULL)) == NULL
      || elf_kind(elf) != ELF_K_ELF
      || gelf_getehdr(elf, &ehdr) == NULL
      || !(ehdr.e_type == ET_DYN || ehdr.e_type == ET_EXEC))
        goto exit;       

    while (!bingo && (scn = elf_nextscn(elf, scn)) != NULL) {
	(void) gelf_getshdr(scn, &shdr);
	if (shdr.sh_type != SHT_DYNAMIC)
	    continue;
	while (!bingo && (data = elf_getdata (scn, data)) != NULL) {
	    int maxndx = data->d_size / shdr.sh_entsize;
	    int ndx;

            for (ndx = 0; ndx < maxndx; ++ndx) {
		(void) gelf_getdyn (data, ndx, &dyn);
		if (!(dyn.d_tag == DT_GNU_PRELINKED || dyn.d_tag == DT_GNU_LIBLIST))
		    continue;
		bingo = 1;
		break;
	    }
	}
    }

exit:
    if (elf) (void) elf_end(elf);
    return Py_BuildValue("i", bingo);
}

static char isprelink_docs[] = "isprelink(fd): Check if the file descriptor is a a prelinked file.  Returns 0 if the file is not prelinked and 1 if the file is prelinked.\n";

static PyMethodDef isprelinkModuleMethods[] = {
    { "isprelink", (PyCFunction)isprelink, METH_KEYWORDS, isprelink_docs },
    {NULL}
};

void
initisprelink(void)
{
   Py_InitModule3("isprelink", isprelinkModuleMethods, "isprelink Extension Module");
}
